use actix::prelude::*;
use actix_web::{server, App, HttpResponse};
use futures::future::Future;
use std::sync::mpsc;
use std::thread;
pub mod websocket;
mod websocketsession;
use websocketsession::ws_route;

pub struct WebServer {
    addr: Addr<actix_net::server::Server>,
    shutdown_rx: mpsc::Receiver<()>,
}

const INDEX: &'static [u8] = include_bytes!("static/index.html");

impl WebServer {
    pub fn run(
        addr: std::net::SocketAddr,
    ) -> (WebServer, futures::sync::mpsc::Sender<websocket::StateUpdate>) {
        let (ws_tx_tx, ws_tx_rx) = mpsc::channel(); // Channel the websocket sender will be sent on
        let (shutdown_tx, shutdown_rx) = mpsc::channel();
        let (tx, rx) = mpsc::channel();
        thread::spawn(move || {
            let sys = actix::System::new("test");

            let wsserver = Arbiter::start(|_| websocket::WSServer::new(ws_tx_tx));
            let ws_tx = ws_tx_rx.recv().unwrap(); // Block waiting to get the websocket update sender
            log::info!("Got tx");

            let addr = server::new(move || {
                App::with_state(websocketsession::WSSessionState {
                    addr: wsserver.clone(),
                })
                .resource("/", |r| {
                    r.f(|_r| HttpResponse::Ok().content_type("text/html").body(INDEX))
                })
                .resource("/ws", |r| r.route().f(ws_route))
            })
            .bind(addr)
            .expect("Error binding to address.")
            .start();

            tx.send((addr, ws_tx)).unwrap();

            sys.run();
            shutdown_tx.send(()).unwrap(); // Notify that we have shutdown
        });

        let (addr, ws_tx) = rx.recv().unwrap();

        (
            WebServer {
                addr: addr,
                shutdown_rx: shutdown_rx,
            },
            ws_tx,
        )
    }

    pub fn shutdown(self) {
        log::debug!("Web server shutting down...");
        self.addr
            .send(actix_web::server::StopServer { graceful: true })
            .wait()
            .unwrap()
            .unwrap();
        let _ = self.shutdown_rx.recv().unwrap(); // wait for webserver shutdown
        log::info!("Web server shutdown");
    }
}
